#!/bin/bash
#
# Installs SAYMON Agent.
# This script should be run under root.
#

THIS_NAME=$0
DIR=$(cd "$( dirname "$0" )" && pwd)
THIS=$DIR/`basename "$0"`

#
# Outputs a log message, prefixed with current date-time.
#
log() {
    echo "[`date`] $1"
}

#
# Outputs a log message without trailing line break.
#
logStart() {
    echo -ne "[`date`] $1"
}

#
# Finishes log entry, previously started with logStart().
#
logEnd() {
    echo "$1"
}

#
# Prints an error message and exits with error.
#
fail() {
    echo -ne "\n[`date`] SAYMON Agent installation failed!\n"

    exit 1
}

#
# Extracts the archive.
#
extract() {
    # Searches where the archive starts.
    SKIP=`awk '/^__Archive_starts_here__/ { print NR + 1; exit 0; }' $THIS`

    logStart "Extracting the archive... "

    # Extract the archive.
    if tail -n +$SKIP "$THIS" | tar -xz; then
        logEnd "OK"
    else
        fail
    fi
}

#
# Saves a given Agent ID to configuration file.
#
# Parameters:
# 1: Agent ID.
#
setAgentId() {
    if [ -n "$1" ]; then
        sed "s/agent.id=.*/agent.id="$1"/g" -i saymon-agent/conf/agent.properties

        log "SAYMON Agent ID set to $1."
    fi
}

# Configuration file re-packaging support.
if [ "$1" = "--repackage" ]; then
    NEW_CONFIG_PATH=$2

    # Fail on any error.
    set -e

    if [ -z "$NEW_CONFIG_PATH" ]; then
        NEW_CONFIG_PATH="/opt/saymon-agent/conf/agent.properties"
    fi

    log "Re-packaging installer with new configuration file: $NEW_CONFIG_PATH"

    extract

    TMP_ARCHIVE=saymon-agent-new.tar.gz

    cp "$NEW_CONFIG_PATH" saymon-agent/conf/agent.properties
    tar -pczf $TMP_ARCHIVE saymon-agent
    mv "$THIS" "$THIS.original"

    # Add missing trailing line break to extract.sh.
    TMP_EXTRACT_SH=`cat saymon-agent/extract.sh`
    echo "$TMP_EXTRACT_SH" > saymon-agent/extract.sh

    saymon-agent/makeself.sh $TMP_ARCHIVE "$THIS_NAME"

    # Clean-up.
    rm $TMP_ARCHIVE
    rm -rf saymon-agent

    log "Installer successfully re-packaged: $THIS"

    exit 0
fi

if [ -z "$1" ]; then
    INTERACTIVE=true
elif [ "$1" = "--auto" ]; then
    log "Using non-interactive (automatic) mode."
    INTERACTIVE=false
else
    log "Agent ID specified ($1), will skip interactive mode."
    AGENT_ID=$1
fi

# Check for superuser privileges.
if [ $EUID -ne 0 ]; then
    echo "This script should be run as root."

    exit 1
fi

# Parameters to build numeric menu.
LIST=1
NUMBER_REGEX='^[0-9]+$'
OPTIONS=(
  "Configure SAYMON Agent during the installation process."
  "Let SAYMON Agent configure itself automatically."
  "Quit without installing SAYMON Agent."
)

# Check if SAYMON Agent is already installed.
if [ -e /opt/saymon-agent ]; then
    log "Looks like SAYMON Agent has been already installed. Please, check /opt/saymon-agent/ directory."

    exit 1
fi

extract

if [ "$INTERACTIVE" = true ]; then
    echo -e "\nPlease select the way you want to configure SAYMON Agent. You will be asked for SAYMON Agent ID," \
        "SAYMON Server IP-address and password.\n "

    # Generate the menu.
    for OPT in "${OPTIONS[@]}"
        do
            echo ""$LIST") $OPT"
            ((LIST++))
        done

    read -e -p ": " INPUT

    if [ $INPUT == 1 ]; then
        while [ -z "$INPUT2" ]
        do
            read -p "Please enter SAYMON Agent ID: " INPUT2

            setAgentId $INPUT2

            if [ -n "$INPUT2" ]; then
                break;
            fi
        done

        while [ -z "$INPUT3" ]
        do
            read -p "Please enter SAYMON Server address (IP or hostname): " INPUT3

            if [ -n "$INPUT3" ]; then
                sed "s/server.host=.*/server.host="$INPUT3"/g" -i saymon-agent/conf/agent.properties
                echo -e "SAYMON Server IP address set to "$INPUT3".\n"
                break;
            fi
        done

        read -p "Please enter SAYMON Server password [press ENTER if password not set] : " INPUT4

        if [ -n "$INPUT4" ]; then
            sed "s/server.password=.*/server.password="$INPUT4"/g" -i saymon-agent/conf/agent.properties
        else
            sed "s/server.password=*./#server.password=password/g" -i saymon-agent/conf/agent.properties
            echo -e " not using any SAYMON Server password.\n"
        fi
    elif [ "$INPUT" == 2 ]; then
        echo -e "SAYMON Agent will attempt automatic configuration. If it fails, you can manually configure SAYMON" \
            "Agent by editing configuration file /opt/saymon-agent/conf/agent.properties. Don't forget to restart" \
            "SAYMON Agent after that.\n"
    elif [ "$INPUT" == 3 ]; then
        logStart "Cancelling installation... "
        rm -rf saymon-agent
        logEnd "OK"

        exit 1
    else
        # Numeric input out of option range => bail out.
        log "Invalid menu entry index (expected number between 1 and ${#OPTIONS[*]})."

        exit 1
    fi
elif [ -n "$AGENT_ID" ]; then
    setAgentId $AGENT_ID
fi

# Move extracted data to /opt/saymon-agent.
logStart "Moving extracted and configured data to /opt/saymon-agent... "
mv saymon-agent /opt/. || fail
logEnd "OK"

# Check if user saymon is already in system, if not - create system user saymon.
if [ -z "`cat /etc/passwd | grep saymon`" ]; then
    logStart "Adding user saymon... "
    groupadd saymon || fail
    adduser --system --no-create-home --shell=/bin/false --gid saymon saymon || fail
    logEnd "OK"
fi

# Assign correct rights.
logStart "Setting ownership to folder /opt/saymon-agent to user saymon... "
chown -R saymon:saymon /opt/saymon-agent || fail
logEnd "OK"

# Install proper daemon script.
if [ -n "`which systemctl`" ]; then
    logStart "Installing SAYMON Agent SystemD service script to /etc/systemd..."
    cp /opt/saymon-agent/systemd/* /etc/systemd/system || fail
    sudo systemctl enable saymon-agent || fail
    logEnd "OK"
elif [ -n "`whereis upstart | awk {'print $2'}`" ]; then
    logStart "Installing SAYMON Agent Upstart service script to /etc/init... "
    cp /opt/saymon-agent/upstart/* /etc/init || fail
    logEnd "OK"
else
    logStart "Installing SAYMON Agent init.d service script to /etc/init.d... "
    cp /opt/saymon-agent/init.d/* /etc/init.d || fail
    logEnd "OK"
fi

# Launch SAYMON Agent service.
logStart "Starting SAYMON Agent service... "
service saymon-agent start > /dev/null || fail
logEnd "OK"

log "SAYMON Agent installation complete!"

exit 0

# Don't add anything below.
__Archive_starts_here__